"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
var cache_validator_1 = require("../bin/cache-validator");
var webpack_client_debug_1 = tslib_1.__importDefault(require("../config/webpack.client.debug"));
var webpack_client_dev_1 = tslib_1.__importDefault(require("../config/webpack.client.dev"));
var webpack_client_prod_1 = tslib_1.__importDefault(require("../config/webpack.client.prod"));
var webpack_server_debug_1 = tslib_1.__importDefault(require("../config/webpack.server.debug"));
var webpack_server_dev_1 = tslib_1.__importDefault(require("../config/webpack.server.dev"));
var webpack_server_prod_1 = tslib_1.__importDefault(require("../config/webpack.server.prod"));
var fs_2 = require("../runtime/util/fs");
exports.TARGETS = ['server', 'client'];
exports.ENVIRONMENTS = ['dev', 'prod', 'debug'];
/**
 * Get the webpack configuration for a given target and environment.
 * This will be the custom configuration if a file is present in
 * config/webpack.<target>.<environment>.js, or the default one otherwise
 */
function getWebpackConfig(log, buildConfig, target, environment) {
    var config = getWebpackConfigInternal(log, buildConfig, target, environment);
    // We need to JSON.strinify config, so let's make it safe (as we can)
    if (config && config.plugins) {
        config.plugins.forEach(function (plugin) {
            var pluginAny = plugin;
            if (pluginAny.toJSON) {
                return;
            }
            pluginAny.toJSON = function () {
                var ctorName = pluginAny.__proto__ &&
                    pluginAny.__proto__.constructor &&
                    pluginAny.__proto__.constructor.name;
                return "Plugin: " + (ctorName || pluginAny.toString());
            };
        });
    }
    return config;
}
exports.getWebpackConfig = getWebpackConfig;
function getWebpackConfigInternal(log, buildConfig, target, environment) {
    if (exports.TARGETS.indexOf(target) === -1) {
        log.error("Unknown target: \"" + target + "\"! " + ("Known values are: " + exports.TARGETS.join(', ')));
        return undefined;
    }
    if (exports.ENVIRONMENTS.indexOf(environment) === -1) {
        log.error("Unknown environment: \"" + environment + "\"! " +
            ("Known values are: " + exports.ENVIRONMENTS.join(', ')));
        return undefined;
    }
    var configFileName = "webpack." + target + "." + environment;
    var customConfigFile = path_1.default.resolve(buildConfig.BASE, 'config', configFileName);
    var cacheDirectory = cache_validator_1.buildCacheDirectory({
        environment: environment,
        project: buildConfig.BASE,
        target: target,
    });
    var config;
    var createWebpackConfigOptions = {
        buildConfig: buildConfig,
        cacheDirectory: cacheDirectory,
        log: log,
    };
    try {
        if (fs_1.default.existsSync(customConfigFile + '.js')) {
            // using dynamicRequire to support bundling project-watchtower with webpack
            var customConfig = fs_2.dynamicRequire(customConfigFile).default;
            config =
                typeof customConfig === 'function'
                    ? customConfig(createWebpackConfigOptions)
                    : customConfig;
            log.info('Using custom config file ' + customConfigFile);
        }
        else {
            log.info('Building ' + configFileName + '...');
            switch (target) {
                case 'server':
                    switch (environment) {
                        case 'dev':
                            return webpack_server_dev_1.default(createWebpackConfigOptions);
                        case 'debug':
                            return webpack_server_debug_1.default(createWebpackConfigOptions);
                        case 'prod':
                            return webpack_server_prod_1.default(createWebpackConfigOptions);
                        default:
                            throw new Error("Invalid build target: " + target + " " + environment);
                    }
                case 'client':
                    switch (environment) {
                        case 'dev':
                            return webpack_client_dev_1.default(createWebpackConfigOptions);
                        case 'debug':
                            return webpack_client_debug_1.default(createWebpackConfigOptions);
                        case 'prod':
                            return webpack_client_prod_1.default(createWebpackConfigOptions);
                        default:
                            throw new Error("Invalid build target: " + target + " " + environment);
                    }
                default:
                    throw new Error("Invalid build target: " + target + " " + environment);
            }
        }
        return config;
    }
    catch (err) {
        log.error({ err: err }, 'Error loading webpack config!');
        return undefined;
    }
}
//# sourceMappingURL=build.js.map