import { __awaiter } from "tslib";
import { formatElapsed, PromiseTracker } from '../../universal';
import { Information, Status404Error, Warning } from './errors';
import { recursiveTaskResolver } from './helpers/recursive-task-resolver';
import { renderApp } from './render-app-to-string';
import { routerContextHandler } from './router-context-handler';
import * as ServerRenderResults from './server-render-results';
export { PromiseTracker };
function renderErrorRoute(ssrRequestProps, options, currentRenderLocation, promiseTracker, err, startTime) {
    return __awaiter(this, void 0, void 0, function* () {
        const errorRenderOptions = err instanceof Status404Error
            ? { location: options.pageNotFoundLocation, status: 404 }
            : { location: options.errorLocation, status: 500 };
        // If we are already rendering the error location, bail
        // This will be caught above and a 500 will be returned
        if (currentRenderLocation === errorRenderOptions.location) {
            throw err;
        }
        // Reset the request state, don't want it for the error route
        options.resetRequest(errorRenderOptions.location);
        // Overwrite the render location with the error location
        currentRenderLocation = errorRenderOptions.location;
        const errorRender = yield renderWithErrorPageFallback(ssrRequestProps, options, errorRenderOptions.location, promiseTracker, startTime);
        if (errorRender.type === ServerRenderResults.ServerRenderResultType.Success) {
            // We have successfully rendered the error page, but it still needs
            // to be an error status code (404 or 500)
            errorRender.statusCode = errorRenderOptions.status;
        }
        return errorRender;
    });
}
function performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime) {
    // We need to capture the store before we render
    // as any changes caused by the render will not be
    // included in the rendered content
    const storeStateAtRenderTime = Object.assign({}, ssrRequestProps);
    try {
        const renderResult = renderApp(renderLocation, options.renderFn, options.log, options.appRender, promiseTracker);
        const result = routerContextHandler(renderResult, startTime, storeStateAtRenderTime, renderLocation);
        return result;
    }
    finally {
        if (options.events && options.events.renderPerformed) {
            try {
                options.events.renderPerformed(promiseTracker);
            }
            catch (err) {
                // External event failed. Just log and continue
                options.log.error({ err }, 'renderPerformed event failed');
            }
        }
    }
}
function renderWithErrorPageFallback(ssrRequestProps, options, renderLocation, promiseTracker, startTime) {
    return __awaiter(this, void 0, void 0, function* () {
        // The inner try is for the initial render
        // if it throws, then the error route will try to render
        // which the second catch is if that route fails
        try {
            const initialRenderResult = performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime);
            // If we have not rendered successfully just return the render result
            if (initialRenderResult.type !== ServerRenderResults.ServerRenderResultType.Success) {
                return initialRenderResult;
            }
            if (options.events && options.events.beginWaitingForTasks) {
                try {
                    options.events.beginWaitingForTasks(formatElapsed(process.hrtime(startTime)));
                }
                catch (err) {
                    // external event failed, log and continue
                    options.log.error({ err, location }, 'beginWaitingForTasks threw, continuing');
                }
            }
            const dataResolved = yield recursiveTaskResolver(options.log, promiseTracker, () => performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime), renderLocation, initialRenderResult, 10, options.ssrTimeoutMs);
            return dataResolved;
        }
        catch (dataLoadErr) {
            if (dataLoadErr instanceof Information) {
                options.log.info({ err: dataLoadErr }, 'Data load warning, rendering error location');
            }
            else if (dataLoadErr instanceof Warning) {
                options.log.warn({ err: dataLoadErr }, 'Data load warning, rendering error location');
            }
            else {
                options.log.error({ err: dataLoadErr }, 'Data load failed, rendering error location');
            }
            return yield renderErrorRoute(ssrRequestProps, options, renderLocation, promiseTracker, dataLoadErr, startTime);
        }
    });
}
export function renderPageContents(ssrRequestProps, options, location, promiseTracker) {
    return __awaiter(this, void 0, void 0, function* () {
        const startTime = process.hrtime();
        try {
            return yield renderWithErrorPageFallback(ssrRequestProps, options, location, promiseTracker, startTime);
        }
        catch (err) {
            const failure = {
                elapsed: formatElapsed(process.hrtime(startTime)),
                errorMessage: 'Failed to do render',
                head: undefined,
                type: ServerRenderResults.ServerRenderResultType.Failure,
            };
            options.log.error({ err }, 'Failed to render');
            return failure;
        }
    });
}
//# sourceMappingURL=full-render.js.map